# 初始化
import os

import django

os.environ.setdefault("DJANGO_SETTINGS_MODULE", "application.settings")
django.setup()

from dvadmin.system.views.menu import MenuInitSerializer
from dvadmin.utils.core_initialize import CoreInitialize


class Initialize(CoreInitialize):

    def init_menu(self):
        """
        初始化菜单信息
        """
        self.init_base(MenuInitSerializer, unique_fields=['name', 'web_path', 'component', 'component_name'])

    def run(self):
        self.init_menu()


if __name__ == "__main__":
    Initialize(app='dvadmin_celery').run()



from application import settings

# 注册app
# app = 'django_apscheduler'
# if app not in settings.INSTALLED_APPS:
#     settings.INSTALLED_APPS += [app]

# app = 'plugins.dvadmin_apscheduler_backend'
# if app not in settings.INSTALLED_APPS:
#     settings.INSTALLED_APPS += [app]

# if getattr(settings, 'PLUGINS_LIST', {}).get('dvadmin_tenant_backend', None):
#     settings.SHARED_APPS = [
#                                'django_apscheduler',
#                                'plugins.dvadmin_apscheduler_backend',
#                            ] + list(getattr(settings, 'SHARED_APPS', []))


# ================================================= #
# ***************** 插件配置区开始 *******************
# ================================================= #
# 路由配置
plugins_url_patterns = [
    {"re_path": r'api/dvadmin_apscheduler/', "include": 'dvadmin_apscheduler.urls'}
]
# app配置
apps = ['django_apscheduler', 'dvadmin_apscheduler']
# 租户模式中，public模式共享app配置
tenant_shared_apps = []
# ================================================= #
# ******************* 插件配置区结束 *****************
# ================================================= #

settings.INSTALLED_APPS += [app for app in apps if app not in settings.INSTALLED_APPS]
settings.TENANT_SHARED_APPS += tenant_shared_apps


# ********** 注册路由 **********
settings.PLUGINS_URL_PATTERNS += plugins_url_patterns



# -*- coding: utf-8 -*-
from django.urls import re_path, path
from rest_framework import routers

from dvadmin_apscheduler.views.task import TaskViewSet
from dvadmin_apscheduler.views.task_detail import TaskDetailViewSet

system_url = routers.SimpleRouter()
system_url.register(r'task', TaskViewSet)

urlpatterns = [
    path('task/job_list/',TaskViewSet.as_view({'get':'job_list'})),
    path('task/update_status/<str:pk>/',TaskViewSet.as_view({'post':'update_status'})),
    path('task_detail/',TaskDetailViewSet.as_view({'get':'list'})),
]
urlpatterns += system_url.urls



# Generated by Django 3.2.3 on 2022-09-21 15:01

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('django_apscheduler', '0009_djangojobexecution_unique_job_executions'),
    ]

    operations = [
        migrations.CreateModel(
            name='Task',
            fields=[
                ('id', models.BigAutoField(help_text='Id', primary_key=True, serialize=False, verbose_name='Id')),
                ('description', models.CharField(blank=True, help_text='描述', max_length=255, null=True, verbose_name='描述')),
                ('modifier', models.CharField(blank=True, help_text='修改人', max_length=255, null=True, verbose_name='修改人')),
                ('dept_belong_id', models.CharField(blank=True, help_text='数据归属部门', max_length=255, null=True, verbose_name='数据归属部门')),
                ('update_datetime', models.DateTimeField(auto_now=True, help_text='修改时间', null=True, verbose_name='修改时间')),
                ('create_datetime', models.DateTimeField(auto_now_add=True, help_text='创建时间', null=True, verbose_name='创建时间')),
                ('is_deleted', models.BooleanField(db_index=True, default=False, help_text='是否软删除', verbose_name='是否软删除')),
                ('status', models.IntegerField(choices=[(0, '停用'), (1, '启用')], default=1, help_text='任务状态', verbose_name='任务状态')),
                ('name', models.CharField(help_text='任务名称', max_length=30, verbose_name='任务名称')),
                ('cron', models.CharField(help_text='cron表达式', max_length=255, verbose_name='cron表达式')),
                ('job', models.CharField(help_text='执行的任务', max_length=255, verbose_name='执行的任务')),
                ('creator', models.ForeignKey(db_constraint=False, help_text='创建人', null=True, on_delete=django.db.models.deletion.SET_NULL, related_query_name='creator_query', to=settings.AUTH_USER_MODEL, verbose_name='创建人')),
                ('job_id', models.OneToOneField(blank=True, db_constraint=False, help_text='关联DjangoJob', null=True, on_delete=django.db.models.deletion.CASCADE, to='django_apscheduler.djangojob', verbose_name='关联DjangoJob')),
            ],
            options={
                'verbose_name': '定时任务表',
                'verbose_name_plural': '定时任务表',
                'db_table': 'dvadmin_apscheduler_task',
                'ordering': ('create_datetime',),
            },
        ),
    ]



from django.apps import AppConfig


class DvadminApschedulerBackendConfig(AppConfig):
    default_auto_field = 'django.db.models.BigAutoField'
    name = 'dvadmin_apscheduler'
    url_prefix = "dvadmin_apscheduler"



# -*- coding: utf-8 -*-

"""
@author: 猿小天
@contact: QQ:1638245306
@Created on: 2021/8/23 023 21:56
@Remark:
"""
import time

from django_apscheduler.models import DjangoJobExecution
from rest_framework import serializers

from dvadmin.utils.serializers import CustomModelSerializer
from dvadmin.utils.viewset import CustomModelViewSet
import django_filters

class  TaskDetailSerializer(CustomModelSerializer):
    """定时任务详情 序列化器"""
    finished = serializers.SerializerMethodField()

    def get_finished(self, instance):
        local = time.localtime(instance.finished)
        local = time.strftime("%Y-%m-%d %H:%M:%S", local)
        return local

    class Meta:
        model = DjangoJobExecution
        fields = "__all__"


class TaskDetailFiterSet(django_filters.FilterSet):
    run_time = django_filters.BaseRangeFilter(field_name="run_time")
    duration = django_filters.NumberFilter(field_name="duration")
    class Meta:
        model = DjangoJobExecution
        fields = "__all__"



class TaskDetailViewSet(CustomModelViewSet):
    """
    定时任务
    """
    queryset = DjangoJobExecution.objects.all()
    serializer_class = TaskDetailSerializer
    filter_class = TaskDetailFiterSet



# -*- coding: utf-8 -*-

"""
@author: 猿小天
@contact: QQ:1638245306
@Created on: 2021/8/22 022 19:23
@Remark:
"""
import uuid

from django.db import models
from django_apscheduler.models import DjangoJob

from application import settings
from dvadmin.utils.models import CoreModel


class Task(CoreModel):
    STATUS_LIST = (
        (0,"停用"),
        (1,"启用")
    )
    status = models.IntegerField(default=1,choices=STATUS_LIST,verbose_name="任务状态",help_text="任务状态")
    name = models.CharField(max_length=30,verbose_name="任务名称",help_text="任务名称")
    cron = models.CharField(max_length=255,verbose_name="cron表达式",help_text="cron表达式")
    job_id = models.OneToOneField(DjangoJob,db_constraint=False,on_delete=models.CASCADE,verbose_name="关联DjangoJob",help_text="关联DjangoJob",null=True,blank=True)
    job = models.CharField(max_length=255,verbose_name="执行的任务",help_text="执行的任务")

    class Meta:
        db_table = settings.TABLE_PREFIX + "apscheduler_task"
        verbose_name = '定时任务表'
        verbose_name_plural = verbose_name
        ordering = ('create_datetime',)


# -*- coding: utf-8 -*-

"""
@author: 猿小天
@contact: QQ:1638245306
@Created on: 2021/8/22 022 10:16
@Remark:
"""
import json
import uuid

from apscheduler.schedulers.background import BackgroundScheduler
from django_apscheduler.jobstores import DjangoJobStore
from dvadmin.utils.json_response import SuccessResponse, ErrorResponse
from dvadmin.utils.serializers import CustomModelSerializer
from dvadmin.utils.viewset import CustomModelViewSet
from dvadmin_apscheduler.models import Task

scheduler = BackgroundScheduler()
scheduler.add_jobstore(DjangoJobStore(), 'default')
scheduler.start()

# 删除任务
# scheduler.remove_job(job_name)
# 暂停任务
# scheduler.pause_job(job_name)
# 开启任务
# scheduler.resume_job(job_name)
# 修改任务
# scheduler.modify_job(job_name)

# 获取所有APP下的tasks任务
def get_job_list():
    from application import settings
    tasks_list = []
    tasks_dict_list=[]
    for app in settings.INSTALLED_APPS:
        try:
            exec(f"""
from {app} import tasks
for ele in [i for i in dir(tasks) if not i.startswith('__')]:
    task_dict = dict()
    task_dict['label']="{app}.tasks." + ele
    task_dict['value']="{app}.tasks." + ele
    tasks_list.append("{app}.tasks." + ele)
    tasks_dict_list.append(task_dict)
                            """)
        except ImportError:
            pass
    return {"tasks_list":tasks_list,"tasks_dict_list":tasks_dict_list}


#将cron表达式进行解析
def CronSlpit(cron):
    cron = cron.split(" ")
    result = {
        "second":cron[0],
        "minute":cron[1],
        "hour":cron[2],
        "day":cron[3],
        "month":cron[4],
        "week":cron[5],
        "year":cron[6]
    }
    return result


class TaskSerializer(CustomModelSerializer):
    """定时任务 序列化器"""
    class Meta:
        model = Task
        fields = "__all__"



class TaskViewSet(CustomModelViewSet):
    queryset = Task.objects.all()
    serializer_class = TaskSerializer
    filter_fields = ['job']

    def job_list(self,request,*args, **kwargs):
        """获取所有任务的路径"""
        result = get_job_list()
        tasks_list = result.get('tasks_dict_list')
        return SuccessResponse(msg="获取成功", data=tasks_list,total=len(tasks_list))

    def create(self, request, *args, **kwargs):
        body_data = request.data
        # {'name': '2222', 'cron': '0/1 0/1 0/1 1/1 1/1 1/1 2022/1', 'job': 'dvadmin_apscheduler.tasks.task2', 'description': '2222'}
        cron = body_data.get('cron')
        cron_lisr = CronSlpit(cron)
        second = cron_lisr["second"]
        minute = cron_lisr["minute"]
        hour = cron_lisr["hour"]
        day = cron_lisr["day"]
        month = cron_lisr["month"]
        week = cron_lisr["week"]
        year = cron_lisr["year"]
        job = body_data.get('job')
        result = None
        tasks_list = get_job_list()
        tasks_list = tasks_list.get('tasks_list')
        if job in tasks_list:
            job_name = job.split('.')[-1]
            path_name = '.'.join(job.split('.')[:-1])
            job_id = uuid.uuid4()
            exec(f"""
from {path_name} import {job_name}
django_job=scheduler.add_job({job_name}, 'cron', id='{job_id}' , second='{second}', minute='{minute}', hour='{hour}', day='{day}', month='{month}', week='{week}', year='{year}')
body_data['job_id'] = django_job.id
serializer = TaskSerializer(data=body_data)
serializer.is_valid(raise_exception=True)
self.perform_create(serializer)
result = serializer.data
            """)
            return SuccessResponse(msg="添加成功", data=result)
        else:
            return ErrorResponse(msg="添加失败,没有该任务", data=None)

    def destroy(self, request, *args, **kwargs):
        """删除定时任务"""
        instance = self.get_object()
        self.perform_destroy(instance)
        job_id = instance.job_id_id
        job = scheduler.pause_job(job_id)
        scheduler.remove_job(job_id)
        return SuccessResponse(data=[], msg="删除成功")

    def update_status(self,request,*args,**kwargs):
        """开始/暂停任务"""
        instance = self.get_object()
        job_id = instance.job_id_id
        body_data = request.data
        status = body_data.get('status')
        if status == 0:
            scheduler.pause_job(job_id)
        else:
            scheduler.resume_job(job_id)
        instance.status = body_data.get('status')
        instance.save()
        return SuccessResponse(msg="修改成功", data=None)


import setuptools

with open("README.md", "r", encoding='utf-8') as fh:
    long_description = fh.read()

setuptools.setup(
    name="dvadmin-apscheduler",
    version="1.0.0",
    author="DVAdmin",
    author_email="liqiang@django-vue-admin.com",
    description="适用于 django-vue-admin 的apscheduler同步插件",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://gitee.com/huge-dream/dvadmin-apscheduler",
    packages=setuptools.find_packages(),
    python_requires='>=3.7, <4',
    install_requires=["django-apscheduler>=0.6.2",
                      "django-redis>=5.0.0",],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    data_files=[
        ('', ['./dvadmin_apscheduler/fixtures/init_menu.json']),
    ],
    packace_data={
        '': ['*.json'],
        'fixtures': ['*.json'],
    },
    include_package_data=True,
)
